-- it is not the movie matrix but the 3D stuff from openGL simplified for 2D

local Matrix={
	temp={},
	orig={},
}

-- make new identity matrix
function Matrix.identity()
	local m={}
	Matrix.setIdentity(m)
	return m
end

-- set identity matrix
function Matrix.setIdentity(m)
	m[1],  m[2],  m[3],  m[4]  = 1, 0, 0, 0
	m[5],  m[6],  m[7],  m[8]  = 0, 1, 0, 0
	m[9],  m[10], m[11], m[12] = 0, 0, 1, 0
	m[13], m[14], m[15], m[16] = 0, 0, 0, 1
end

-- update camera view matrix
function Matrix.updateView(m,x,y,z)
	m[4]=x
	m[8]=y
	m[12]=z
end

-- update camera projection matrix
function Matrix.updateProjection(m,fov,near,far,aspectRatio)
	local top=near*math.tan(fov/2)
	local bottom=-1*top
	local right=top*aspectRatio
	local left=-1*right

	m[1],  m[2],  m[3],  m[4]  = 2*near/(right-left), 0, (right+left)/(right-left), 0
	m[5],  m[6],  m[7],  m[8]  = 0, 2*near/(top-bottom), (top+bottom)/(top-bottom), 0
	m[9],  m[10], m[11], m[12] = 0, 0, -1*(far+near)/(far-near), -2*far*near/(far-near)
	m[13], m[14], m[15], m[16] = 0, 0, -1, 0
end

-- update camera ortographic projection matrix
function Matrix.updateOrtographic(m,fov,size,near,far,aspectRatio)
	local top=size*math.tan(fov/2)
	local bottom=-1*top
	local right=top*aspectRatio
	local left=-1*right

	m[1],  m[2],  m[3],  m[4]  = 2/(right-left), 0, 0, -1*(right+left)/(right-left)
	m[5],  m[6],  m[7],  m[8]  = 0, 2/(top-bottom), 0, -1*(top+bottom)/(top-bottom)
	m[9],  m[10], m[11], m[12] = 0, 0, -2/(far-near), -(far+near)/(far-near)
	m[13], m[14], m[15], m[16] = 0, 0, 0, 1
end

-- copy matrix from m1 to m2
function Matrix.copy(m1,m2)
	for i=1,16 do
		m2[i]=m1[i]
	end
end

-- return value from m at x,y
function Matrix.getValueAt(m,x,y)
	return m[x+(y-1)*4]
end

-- multiple two matrixes and store the result into m1
function Matrix.multiply(m1,m2)
	-- original matrix stored
	local orig=Matrix.orig
	Matrix.copy(m1,orig)

	-- multiply m1 by m2
	local i = 1
	for y=1, 4 do
		for x=1, 4 do
			m1[i] = Matrix.getValueAt(orig,1,y)*Matrix.getValueAt(m2,x,1)
			m1[i] = m1[i] + Matrix.getValueAt(orig,2,y)*Matrix.getValueAt(m2,x,2)
			m1[i] = m1[i] + Matrix.getValueAt(orig,3,y)*Matrix.getValueAt(m2,x,3)
			m1[i] = m1[i] + Matrix.getValueAt(orig,4,y)*Matrix.getValueAt(m2,x,4)
			i = i + 1
		end
	end
end

-- update object matrix
function Matrix.updateTransformation(m,position,rotation,scale)
	-- temporal matrix
	local temp=Matrix.temp
	
	-- set it to identity
	Matrix.setIdentity(m)

	-- position
	m[4]=position[1]
	m[8]=position[2]
	m[12]=position[3]

	-- rotations
	-- use 3D rotation vector as euler angles
	-- x
	Matrix.setIdentity(temp)
	temp[6] = math.cos(rotation[1])
	temp[7] = -1*math.sin(rotation[1])
	temp[10] = math.sin(rotation[1])
	temp[11] = math.cos(rotation[1])
	Matrix.multiply(m,temp)

	-- y
	Matrix.setIdentity(temp)
	temp[1] = math.cos(rotation[2])
	temp[3] = math.sin(rotation[2])
	temp[9] = -1*math.sin(rotation[2])
	temp[11] = math.cos(rotation[2])
	Matrix.multiply(m,temp)

	-- z
	Matrix.setIdentity(temp)
	temp[1] = math.cos(rotation[3])
	temp[2] = -1*math.sin(rotation[3])
	temp[5] = math.sin(rotation[3])
	temp[6] = math.cos(rotation[3])
	Matrix.multiply(m,temp)

	-- scale
	Matrix.setIdentity(temp)
	temp[1] = scale[1]
	temp[6] = scale[2]
	temp[11] = scale[3]
	Matrix.multiply(m,temp)
end

return Matrix