-- ok, this should be the textured polygon as triangle model

local Polygon={
	-- format of vertex for the GLSL shader
	vertexFormat = {
    {"VertexPosition", "float", 3},
    {"VertexTexCoord", "float", 2},
    {"VertexNormal", "float", 3},
    {"VertexColor", "byte", 4},
	},
}

-- make a polygon for testing the feature
function Polygon.test(imagePath,colorize)
	--local imagePath="data/rock.png"
	
	-- i make just one quad from 2 triangles
	local x1=10
	local x2=-10
	local y1=10
	local y2=-10
	
	-- the texture will be repeated 4x4 tiles inside the quad
	local u1=0
	local v1=0
	local u2=4
	local v2=4
	
	-- use format x,y,z, u,v, nx,ny,nz, r,g,b,a
	-- where
	-- x,y,z		= vertex point where z is always 0 because this is not supposed to be a 3D game
	-- u,v			= texture UV coordinate
	-- nx,ny,nz	= normal always 0,0,-1 to face the camera
	-- r,g,b,a	= per vertex color where a is always 1, but when you do not set anything it will be white 1,1,1,1
	-- if you get the order of vertices wrong the triangle will dissappear
	-- when that happens look at window.lua and set the culling method to from "back" to "none" in window.lua
	local verts
	
	if colorize then
		verts={
			-- front face triangle 1
			{x2,y2,0, u2,v2, 0,0,-1, 1,0,0,1},
			{x1,y1,0, u1,v1, 0,0,-1, 1,1,0,1},		
			{x2,y1,0, u2,v1, 0,0,-1, 0,1,0,1},
			-- front face triangle 2
			{x2,y2,0, u2,v2, 0,0,-1, 1,0,0,1},
			{x1,y2,0, u1,v2, 0,0,-1, 0,1,1,1},
			{x1,y1,0, u1,v1, 0,0,-1, 1,1,0,1},
		}
	else
		verts={
			-- front face triangle 1
			{x2,y2,0, u2,v2, 0,0,-1},
			{x1,y1,0, u1,v1, 0,0,-1},		
			{x2,y1,0, u2,v1, 0,0,-1},
			-- front face triangle 2
			{x2,y2,0, u2,v2, 0,0,-1},
			{x1,y2,0, u1,v2, 0,0,-1},
			{x1,y1,0, u1,v1, 0,0,-1},
		}
	end
	
	-- the resulting polygon table
	local polygon=Polygon.make(imagePath,verts)
	return polygon
end

-- make an model from image and vertices
function Polygon.make(imagePath,verts)
	-- load textue from file
	local image=Resource.getImage(imagePath)
	
	-- make a model out of vertices and image
	local model=love.graphics.newMesh(Polygon.vertexFormat,verts,"triangles")
	model:setTexture(image)
	
	-- and this will be the new polygon made out of triangles
	local polygon={
		-- each polygon has its own values that define it's position for X,Y then rotation for Z and scale for X,Y
		position={0,0,0},
		rotation={0,0,0},
		scale={1,1,1},
		
		-- the matrix is an calculation based on position, rotation and scale that will be send to the GLSL shader
		matrix=Matrix.identity(),
		
		-- the texture stored also only as reference
		image=image,
		
		-- vertices stored only for editation
		verts=verts,
		
		-- compiled vertices and texture as one single model pointer
		model=model,
	}
	
	return polygon
end

-- draw the textured polygon model
function Polygon.draw(shader,polygon)
	Shader.send(shader,"modelMatrix",polygon.matrix)
	love.graphics.draw(polygon.model)
end

-- set position of polygon
function Polygon.setPosition(polygon,x,y)
	polygon.position[1]=x
	polygon.position[2]=y
end

-- set rotation of polygon
function Polygon.setRotation(polygon,z)
	polygon.rotation[3]=z
end

-- set scale of polygon
function Polygon.setScale(polygon,x,y)
	polygon.scale[1]=x
	polygon.scale[2]=y
end

-- update polygon transormation matrix when position, rotation or scale change
-- note : do not do this in love.draw call, only in love.update call
function Polygon.updateMatrix(polygon)
	Matrix.updateTransformation(polygon.matrix,polygon.position,polygon.rotation,polygon.scale)
end

return Polygon