-- basic shader

local Shader={}

-- return default vertex and fragment GLSL shader program, no fancy lighting or other kind of like effects included
function Shader.getDefault()
	local shader = love.graphics.newShader [[
    uniform mat4 projectionMatrix;
    uniform mat4 modelMatrix;
    uniform mat4 viewMatrix;

    varying vec4 vertexColor;

    #ifdef VERTEX
        vec4 position(mat4 transform_projection, vec4 vertex_position)
        {
            vertexColor = VertexColor;
            return projectionMatrix * viewMatrix * modelMatrix * vertex_position;
        }
    #endif

    #ifdef PIXEL
        vec4 effect(vec4 color, Image tex, vec2 texcoord, vec2 pixcoord)
        {
            vec4 texcolor = Texel(tex, vec2(texcoord.x, 1.0-texcoord.y));
            if (texcolor.a == 0.0) { discard; }
            return vec4(texcolor)*color*vertexColor;
        }
    #endif
	]]
	return shader
end

-- apply GLSL shader program to GPU
function Shader.apply(shader)
	love.graphics.setShader(shader)
end

-- send GLSL variable and value to shader program
function Shader.send(shader,variable,value)
	shader:send(variable,value)
end

return Shader